Crate uv_once_map

Crate uv_once_map 

Structs

OnceMap
Run tasks only once and store the results in a parallel hash map.
UnregisteredTask
The caller tried to wait for a task that was never registered.